import winston from 'winston';
import { check, MessageType, MiraiApiHttpSetting, Contact, MiraiInstance, Api } from 'mirai-ts';
import commander from 'commander';
import mongoose from 'mongoose';
import events from 'events';
import { Server } from 'net';
import Koa from 'koa';
import { Webhooks } from '@octokit/webhooks';
import { CustomFields } from 'rss-parser';
import nodeSchdule from 'node-schedule';

interface Logger extends winston.Logger {
    success: winston.LeveledLogMethod;
}
declare function createLogger(label?: string): Logger;

interface WebhookConfig {
    enable?: boolean;
    port?: number;
    path?: string;
    secret?: string;
    callback?: (webhook: any) => {};
}
declare class Webhook {
    ctx: Bot;
    config: WebhookConfig;
    emitter: events.EventEmitter;
    githubHandler: Webhooks<any>;
    middleware: (request: any, response: any, next?: Function | undefined) => Promise<any>;
    constructor(ctx: Bot);
    start(config?: WebhookConfig): Server | undefined;
    parse(ctx: Koa.ParameterizedContext<Koa.DefaultState, Koa.DefaultContext>): void;
    on(type: string, callback: Function): void;
}

interface Target {
    friend?: number[];
    group?: number[];
}
declare type BaseListenType = 'all' | 'master' | 'admin' | 'friend' | 'group';
declare type Listen = Target | (BaseListenType | number)[];

interface BaseAnswerOptions extends check.Match {
    listen?: string | Listen;
    unlisten?: Listen;
    cron?: nodeSchdule.RecurrenceRule;
    target?: Target;
    api?: string;
    reply: string | MessageType.MessageChain;
    at?: boolean;
    quote?: boolean;
    else?: string | MessageType.MessageChain;
    help?: string;
}
declare type AnswerOptions = BaseAnswerOptions[];

interface ForwardItem {
    listen: Listen;
    target: Target;
}
declare type ForwardOptions = ForwardItem[];

interface RssConfig {
    name: string;
    url: string;
    cron: string;
    customFields: CustomFields<string[], string[]>;
    content: string[];
    target: object;
}
declare type RssOptions = RssConfig[];

interface BotConfig {
    name: string;
    autoloadPlugins: boolean;
    pluginDir: string;
    plugins: {
        default?: string[];
        official?: string[];
        community?: string[];
        custom?: string[];
    };
    master: number[];
    admin: number[];
    devGroup: number;
    answer?: AnswerOptions;
    forward?: ForwardOptions;
    rss?: RssOptions;
    [propName: string]: any;
}
declare type BotUserConfig = Partial<BotConfig>;

interface dbConfig {
    enable: boolean;
    uri?: string;
    analytics?: Boolean;
}
interface reportConfig {
    enable: boolean;
    target?: Target;
}
declare class El {
    qq: number;
    setting: MiraiApiHttpSetting | string;
    db?: dbConfig;
    bot: BotConfig;
    webhook?: WebhookConfig;
    report?: reportConfig;
    pkg?: any;
    base?: string | undefined;
    path?: {
        image: string;
        voice: string;
    } | undefined;
    constructor(el: El);
}

declare class Sender {
    ctx: Bot;
    constructor(ctx: Bot);
    sendFriendMessageByArray(messageChain: string | MessageType.MessageChain, array: number[], messageList: number[]): Promise<void[]>;
    sendMessageByConfig(messageChain: string | MessageType.MessageChain, target: Target): Promise<number[]>;
}

declare class User {
    ctx: Bot;
    constructor(ctx: Bot);
    isMaster(qq: number): boolean;
    isAdmin(qq: number): boolean;
    isAllowed(qq?: number, reply?: boolean, content?: string): boolean;
}

declare class Status {
    ctx: Bot;
    constructor(ctx: Bot);
    isListening(sender: Contact.User, listen: Listen): boolean;
    getListenStatusByConfig(sender: Contact.User, config: any): boolean;
}

declare type PluginInstallFunction = (ctx: Bot, ...options: any[]) => any;
declare type Plugin = (PluginInstallFunction & {
    name?: string;
    version?: string;
    description?: string;
    install?: PluginInstallFunction;
}) | {
    install: PluginInstallFunction;
};
interface PluginInfo {
    name: string;
    version?: string;
    description?: string;
}
declare type PluginType = 'default' | 'official' | 'community' | 'custom';
declare class Plugins {
    ctx: Bot;
    default: Set<PluginInfo>;
    official: Set<PluginInfo>;
    community: Set<PluginInfo>;
    custom: Set<PluginInfo>;
    constructor(ctx: Bot);
    isOfficial(name: string): boolean;
    isCommunity(name: string): boolean;
    getPluginFullName(name: string, type: PluginType): string;
    load(type: PluginType): void;
    isBasedOnDb(pkg: any): boolean;
    add(name: string, plugin: Plugin, options?: any, pkg?: any): void;
    list(type: PluginType): string;
}

declare class Command {
    ctx: Bot;
    name: string;
    desc: string;
    callback: (options: string[]) => void;
    children: Map<string, Command>;
    constructor(ctx: Bot);
    command(name: string): this;
    description(desc: string): this;
    action(callback: (options: string[]) => any): this;
    parse(text: string): void;
    listen(): void;
}

declare function createBot(el: El): Bot;
declare class Bot {
    el: El;
    mirai: MiraiInstance;
    active: boolean;
    db?: mongoose.Connection;
    status: Status;
    user: User;
    sender: Sender;
    plugins: Plugins;
    installedPlugins: Set<unknown>;
    cli: commander.Command;
    _command: Command;
    logger: Logger;
    webhook?: Webhook;
    isDev: boolean;
    rootDir: string;
    tmpDir: string;
    isTS: boolean;
    constructor(el: El);
    reply(msgChain: string | MessageType.MessageChain, quote?: boolean): false | Promise<void>;
    link(): Promise<Api.Response.BaseResponse | undefined>;
    start(): Promise<void>;
    use(plugin: Plugin, ...options: any[]): this;
    plugin(name: string, plugin: Plugin, ...options: any[]): void;
    command(name: string): Command;
}

declare function loadConfigFromFile(configFile?: string, configRoot?: string): Promise<BotConfig | null | undefined>;
declare function defineConfig(config: El): El;
declare function defineBotConfig(config: BotUserConfig): BotUserConfig;

declare function parse(path: string): unknown;
declare function merge(target: any, source: any): any;

declare const config_parse: typeof parse;
declare const config_merge: typeof merge;
declare namespace config {
  export {
    config_parse as parse,
    config_merge as merge,
  };
}

declare function isUrl(url: string): boolean;
declare class InnerMode {
    msg: MessageType.ChatMessage | undefined;
    friendSet: Set<unknown>;
    groupSet: Set<unknown>;
    constructor(msg?: MessageType.ChatMessage);
    setMsg(msg: MessageType.ChatMessage): void;
    enter(): void;
    getStatus(): boolean | undefined;
    exit(): void;
}

declare function renderString(template: string, data: any, name: string): any;

declare const index_config: typeof config;
declare const index_isUrl: typeof isUrl;
type index_InnerMode = InnerMode;
declare const index_InnerMode: typeof InnerMode;
declare const index_renderString: typeof renderString;
declare namespace index {
  export {
    index_config as config,
    index_isUrl as isUrl,
    index_InnerMode as InnerMode,
    index_renderString as renderString,
  };
}

export { Bot, BotConfig, El, createBot, createLogger, Bot as default, defineBotConfig, defineConfig, loadConfigFromFile, index as utils };
